__version__ = '0.5.7'
__author__ = "Dave Martorana (http://davemartorana.com), Richard Cooper (http://frozenskys.com), Bill Jones (oraclebill), Dmitry Golomidov (deeGraYve)"
__date__ = '2010-April-14'
__url__ = 'http://postmarkapp.com'
__copyright__ = "(C) 2009-2021 David Martorana, Wildbit LLC, Python Software Foundation."
__doc__ = '''

    PMMail object for Postmark (http://postmarkapp.com)

    Version: ''' + __version__ + '''
    Author: ''' + __author__ + '''
    Last Updated: ''' + __date__ + '''

    USAGE:
        Make sure you have a Postmark account.  Visit
        http://postmarkapp.com to sign up for an account.
        Requires a Postmark API key.

        Import postmark.PMMail to use Postmark Sending.
        Check class documentation on PMMail object for
        more information.

        Import postmark.PMBatchMail object to send batches of
        messages. Either pass a "messages" argument or set
        the .messages property of the PMBatchMail object to an
        array of PMMail objects.

        Import postmark.PMBounceManager to use Postmark Bounce API.
        Check class documentation on PMBounceManager object for
        more information.

    DJANGO:
        The library can be used stand-alone with Django.  You can also
        add the setting

        POSTMARK_API_KEY = 'your-key'
        POSTMARK_SENDER = '<From Name> from@emailaddress.com'
        POSTMARK_TEST_MODE = True/False

        to your settings.py file, and when you create a new PMMail object,
        it will grab the API key automatically.

        Using POSTMARK_TEST_MODE=True will not actually send the email, but
        instead dump the JSON packet that would be sent to Postmarkapp.com.
        By default this setting is False, and if not specified, will
        be assumed to be False.

        To reroute all Django E-Mail functions like send_mail() and
        mail_admins() through postmark use the following setting:

        EMAIL_BACKEND = 'postmark.django_backend.EmailBackend'

        But keep in mind that even when using standard Django functions
        the sender must be registered with postmarkapp.com.

    EXCEPTIONS:
        PMMailMissingValueException(Exception):
            One of the required values for attempting a send request is missing

        PMMailSendException(Exception):
            Base Postmark send exception

        PMMailUnauthorizedException(PMMailSendException):
            401: Unauthorized sending due to bad API key

        PMMailUnprocessableEntityException(PMMailSendException):
            422: Unprocessable Entity - usually an exception with either the sender
            not having a matching Sender Signature in Postmark.  Read the message
            details for further information

        PMMailServerErrorException(PMMailSendException):
            500: Internal error - this is on the Postmark server side.  Errors are
            logged and recorded at Postmark.

        PMMailURLException(PMMailSendException):
            A URLError was caught - usually has to do with connectivity
            and the ability to reach the server.  The inner_exception will
            have the base URLError object.

        PMMailInactiveRecipientException(PMMailSendException):
            406: You tried to send a message to a recipient that has been marked as
            inactive. If this was a batch operation, the rest of the messages were
            still sent.

    TODO:
        Add automatic multipart emails via regex stripping of HTML tags from html_body
        if the .multipart property is set to True

'''

from postmark.core import *



"""
Postmark Mixin
"""
from tornado.options import options as opt
import postmark

class PostmarkMixin(object):
    def send_email(self, **kwargs):
        subject=kwargs.get('subject')
        body=kwargs.get('body')
        to=kwargs.get('to')
        postsig = opt.postmark_signature
        postemail = opt.postmark_sendemail

        sender = postmark.PMMail()
        sender.sender = postemail
        sender.api_key = postsig
        sender.to = to
        sender.html_body = body
        sender.subject = subject
        sender.send(test=False)



from distutils.core import setup
from distutils.util import convert_path

postmark = {}
ver_path = convert_path('postmark/__init__.py')
with open(ver_path) as ver_file:
    exec(ver_file.read(), postmark)

setup(
    name="python-postmark",
    version=postmark['__version__'],
    packages=['postmark'],
    author="Dave Martorana (http://davemartorana.com), Richard Cooper (http://frozenskys.com), Bill Jones (oraclebill), Dmitry Golomidov (deeGraYve)",
    author_email="themartorana@gmail.com",
    license='BSD',
    description="Postmark library for Python 2.7 and greater.",
    long_description="Note: To use Python 2.4 or 2.5, please use python-postmark version 0.3.2. Python 2.6 use version 0.4.4.",
    url="http://github.com/themartorana/python-postmark",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Communications :: Email',
    ]
)



from django.conf import settings
from django.core.mail.backends.base import BaseEmailBackend
from django.core.exceptions import ImproperlyConfigured
from django.core.mail import EmailMessage, EmailMultiAlternatives
import base64

from postmark.core import PMMail, PMBatchMail


class PMEmailMessage(EmailMessage):
    def __init__(self, *args, **kwargs):
        if 'tag' in kwargs:
            self.tag = kwargs['tag']
            del kwargs['tag']
        else:
            self.tag = None

        if 'track_opens' in kwargs:
            self.track_opens = kwargs['track_opens']
            del kwargs['track_opens']
        else:
            self.track_opens = getattr(settings, 'POSTMARK_TRACK_OPENS', False)

        if 'message_stream' in kwargs:
            self.message_stream = kwargs['message_stream']
            del kwargs['message_stream']
        else:
            self.message_stream = None

        super(PMEmailMessage, self).__init__(*args, **kwargs)


class PMEmailMultiAlternatives(EmailMultiAlternatives):
    def __init__(self, *args, **kwargs):
        if 'tag' in kwargs:
            self.tag = kwargs['tag']
            del kwargs['tag']
        else:
            self.tag = None

        if 'track_opens' in kwargs:
            self.track_opens = kwargs['track_opens']
            del kwargs['track_opens']
        else:
            self.track_opens = getattr(settings, 'POSTMARK_TRACK_OPENS', False)

        if 'message_stream' in kwargs:
            self.message_stream = kwargs['message_stream']
            del kwargs['message_stream']
        else:
            self.message_stream = None


        super(PMEmailMultiAlternatives, self).__init__(*args, **kwargs)


class EmailBackend(BaseEmailBackend):

    def __init__(self, api_key=None, default_sender=None, **kwargs):
        """
        Initialize the backend.
        """
        super(EmailBackend, self).__init__(**kwargs)
        self.api_key = api_key if api_key is not None else getattr(settings, 'POSTMARK_API_KEY', None)
        if self.api_key is None:
            raise ImproperlyConfigured('POSTMARK API key must be set in Django settings file or passed to backend constructor.')
        self.default_sender = getattr(settings, 'POSTMARK_SENDER', default_sender)
        self.test_mode = getattr(settings, 'POSTMARK_TEST_MODE', False)
        self.return_message_id = getattr(settings, 'POSTMARK_RETURN_MESSAGE_ID', False)

    def send_messages(self, email_messages):
        """
        Sends one or more EmailMessage objects and returns the number of email
        messages sent.
        """
        if not email_messages:
            return

        sent, instance = self._send(email_messages)

        if sent and self.return_message_id:
            return [m.message_id for m in instance.messages]
        elif sent:
            return len(email_messages)
        return 0

    def _build_message(self, message):
        """A helper method to convert a PMEmailMessage to a PMMail"""
        if not message.recipients():
            return False
        recipients = ','.join(message.to)
        recipients_cc = ','.join(message.cc)
        recipients_bcc = ','.join(message.bcc)

        text_body = message.body
        html_body = None
        if isinstance(message, EmailMultiAlternatives):
            for alt in message.alternatives:
                if alt[1] == "text/html":
                    html_body = alt[0]
                    break

        if getattr(message, 'content_subtype', None) == 'html':
            # Don't send html content as plain text
            text_body = None
            html_body = message.body

        reply_to = ','.join(message.reply_to)
        custom_headers = {}
        if message.extra_headers and isinstance(message.extra_headers, dict):
            if 'Reply-To' in message.extra_headers:
                reply_to = message.extra_headers.pop('Reply-To')
            if len(message.extra_headers):
                custom_headers = message.extra_headers
        attachments = []
        if message.attachments and isinstance(message.attachments, list):
            if len(message.attachments):
                for item in message.attachments:
                    if isinstance(item, tuple):
                        (f, content, m) = item
                        if isinstance(content, str):
                            content = content.encode()
                        content = base64.b64encode(content)
                        # b64decode returns bytes on Python 3. PMMail needs a
                        # str (for JSON serialization). Convert on Python 3
                        # only to avoid a useless performance hit on Python 2.
                        if not isinstance(content, str):
                            content = content.decode()
                        attachments.append((f, content, m))
                    else:
                        attachments.append(item)

        message_stream = getattr(message, 'message_stream', None)
        postmark_message = PMMail(api_key=self.api_key,
                                  subject=message.subject,
                                  sender=message.from_email,
                                  to=recipients,
                                  cc=recipients_cc,
                                  bcc=recipients_bcc,
                                  text_body=text_body,
                                  html_body=html_body,
                                  reply_to=reply_to,
                                  custom_headers=custom_headers,
                                  attachments=attachments,
                                  message_stream=message_stream)

        postmark_message.tag = getattr(message, 'tag', None)
        postmark_message.track_opens = getattr(message, 'track_opens', False)

        return postmark_message

    def _send(self, messages):
        """A helper method that does the actual sending."""
        if len(messages) == 1:
            to_send = self._build_message(messages[0])
            if to_send is False:
                # The message was missing recipients.
                # Bail.
                return False
        else:
            pm_messages = list(map(self._build_message, messages))
            pm_messages = [m for m in pm_messages if m]
            if len(pm_messages) == 0:
                # If after filtering, there aren't any messages
                # to send, bail.
                return False
            to_send = PMBatchMail(messages=pm_messages)
        try:
            to_send.send(test=self.test_mode)
        except:
            if self.fail_silently:
                return False, to_send
            raise
        return True, to_send
